/*  Test capacitive touch sensor function on Teensy 3.0
 using Teensy 3.0 beta 7 (Windows) release J.Beale Nov. 7 2012
*/

 #define SAMPLES (5000)   // maximum number of separate readings to take
 #define TIMEOUT (500) // maximum sample integration time in milliseconds
 #define TCHANS 12  // number of touch channels to read out

int led = 13;
const int tchan[] = {0,1,15,16,17,18,19,22,23,25,32,33};

void setup() {      
 int i, chan;
 
   Serial.begin(115200);    // baud rate is ignored with Teensy USB ACM i/o
   pinMode(led, OUTPUT);  
   digitalWrite(led, HIGH);   // turn the LED on
   delay(8000);      // wait for user to start up logging program
   for (i=0; i<TCHANS; i++) {  // step through all channels
     chan = tchan[i];  
     if (i!=0)  Serial.print(",");
     Serial.print("Avg"); Serial.print(chan); Serial.print(",");
     Serial.print("PPk"); Serial.print(chan); Serial.print(",");
     Serial.print("Std"); Serial.print(chan);
     digitalWrite(tchan[i],LOW);
   }
   Serial.println();

   Serial.println("# Teensy 3.0 Touch program Oct. 23 2012");
   digitalWrite(led, LOW); // turn the LED off
}  // end setup()

 void loop() {
 int i,j;  // generic loop variables
 int chan; // touch-sense pin number
 int t1;
 long tStart;  // starting time
 long datSum;  // reset our accumulated sum of input values to zero
 int sMax;
 int sMin;
 long n;   // count of how many readings so far
 double x,mean,delta,sumsq,m2,variance,stdev;  // to calculate standard deviation

 for (j=0; j < TCHANS; j++) {
   chan = tchan[j];  
   for (i=0; i<TCHANS; i++) {  // first, set all analog input pins to Output: LOW level    
     pinMode(tchan[i],OUTPUT);
     digitalWrite(tchan[i],LOW);
   }
   datSum = 0;
   sumsq = 0; // initialize running squared sum of differences
   n = 0;  // have not made any ADC readings yet
   mean = 0; // start off with running mean at zero
   m2 = 0;
   sMax = 0;
   sMin = 65535;
   tStart = millis();
   pinMode(chan,INPUT);  // set the active touch-read pin to be an input
   for (i=0;i<SAMPLES && ((millis()-tStart)<TIMEOUT);i++) {
    x = touchRead(chan);
    datSum += x;
    if (x > sMax) sMax = x;
    if (x < sMin) sMin = x;
       // from http://en.wikipedia.org/wiki/Algorithms_for_calculating_variance
    n++;
    delta = x - mean;
    mean += delta/n;
    m2 += (delta * (x - mean));
     }
   variance = m2/(n-1);  // (n-1):Sample Variance  (n): Population Variance
   stdev = sqrt(variance);  // Calculate standard deviation

   if (j!=0) Serial.print(","); 
   Serial.print(mean);
   Serial.print(","); 
//   Serial.print(n);
//   Serial.print(","); 
   Serial.print(sMax-sMin);
   Serial.print(","); 
   Serial.print(stdev,3);
   }
 Serial.println();   // end of this data line
} // end loop()
